package com.tencent.cloud.dtf.demo.inventory.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Component;

import com.tencent.cloud.dtf.demo.inventory.Inventory;

@Component
public class InventoryDao {

    private static final Logger LOG = LoggerFactory.getLogger(InventoryDao.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public boolean deduct(Inventory inventory) {
        String sql = "UPDATE dtf_demo_inventory SET frozen = frozen + ? WHERE product_id = ? AND qty >= frozen + ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, inventory.getQty());
                    ps.setInt(2, inventory.getProductId());
                    ps.setInt(3, inventory.getQty());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Create inventory failed.", e);
            throw new RuntimeException("Create inventory failed.");
        }
    }

    public boolean confirmInventory(Inventory inventory) {
        String sql = "UPDATE dtf_demo_inventory SET frozen = frozen - ?, qty = qty - ? WHERE product_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, inventory.getQty());
                    ps.setInt(2, inventory.getQty());
                    ps.setInt(3, inventory.getProductId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Confirm inventory failed.", e);
            throw new RuntimeException("Confirm inventory failed.");
        }
    }

    public boolean cancelInventory(Inventory inventory) {
        String sql = "UPDATE dtf_demo_inventory SET frozen = frozen - ? WHERE product_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, inventory.getQty());
                    ps.setInt(2, inventory.getProductId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Cancel inventory failed.", e);
            throw new RuntimeException("Cancel inventory failed.");
        }
    }

    public boolean createInventoryTmp(Long txId, Long branchId, Inventory inventory) {
        String sql = "INSERT INTO dtf_demo_inventory_tmp (tx_id, branch_id, product_id, qty) VALUES (?, ?, ?, ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                    ps.setLong(3, inventory.getProductId());
                    ps.setInt(4, inventory.getQty());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Create inventory tmp failed.", e);
            throw new RuntimeException("Create inventory tmp failed.");
        }
    }

    public boolean deleteInventoryTmp(Long txId, Long branchId) {
        String sql = "DELETE FROM dtf_demo_inventory_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Delete inventory tmp failed.", e);
            throw new RuntimeException("Delete inventory tmp failed.");
        }
    }

    public boolean queryInventoryTmp(Long txId, Long branchId) {
        String sql = "SELECT product_id, qty FROM dtf_demo_inventory_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.query(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }, new ResultSetExtractor<Boolean>() {
                @Override
                public Boolean extractData(ResultSet rs) throws SQLException, DataAccessException {
                    return null != rs && rs.next();
                }
            });
        } catch (Exception e) {
            LOG.error("Query inventory tmp failed.", e);
            throw new RuntimeException("Query inventory tmp failed.");
        }
    }
}
