package com.tencent.cloud.dtf.demo.inventory.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.tencent.cloud.dtf.demo.inventory.Inventory;
import com.tencent.cloud.dtf.demo.inventory.dao.InventoryDao;
import com.tencent.cloud.dtf.demo.inventory.service.IInventoryService;

@Service
public class InventoryService implements IInventoryService {

    @Autowired
    private InventoryDao inventoryDao;

    @Transactional
    @Override
    public boolean deduct(Long txId, Long branchId, Inventory inventory) {
        if (inventoryDao.deduct(inventory) && inventoryDao.createInventoryTmp(txId, branchId, inventory)) {
            return true;
        }
        throw new RuntimeException("Deduct inventory failed.");
    }

    @Transactional
    @Override
    public boolean confirmDeduct(Long txId, Long branchId, Inventory inventory) {
        if (inventoryDao.queryInventoryTmp(txId, branchId)) {
            if (inventoryDao.confirmInventory(inventory) && inventoryDao.deleteInventoryTmp(txId, branchId)) {
                return true;
            }
            // 使用异常对事务进行回滚
            throw new RuntimeException("Confirm inventory failed.");
        } else {
            // 幂等
            return true;
        }
    }

    @Transactional
    @Override
    public boolean cancelDeduct(Long txId, Long branchId, Inventory inventory) {
        if (inventoryDao.queryInventoryTmp(txId, branchId)) {
            if (inventoryDao.cancelInventory(inventory) && inventoryDao.deleteInventoryTmp(txId, branchId)) {
                return true;
            }
            // 使用异常对事务进行回滚
            throw new RuntimeException("Cancel inventory failed.");
        } else {
            // 幂等
            return true;
        }
    }
}
