package com.tencent.cloud.dtf.demo.order.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Component;

import com.tencent.cloud.dtf.demo.order.Order;

@Component
public class OrderDao {
    private static final Logger LOG = LoggerFactory.getLogger(OrderDao.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public int createOrder(Order order) {
        String sql = "INSERT INTO dtf_demo_order (product_id, qty, account_id) VALUES (?, ?, ?)";
        try {
            GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
            jdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection con) throws SQLException {
                    PreparedStatement ps = con.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
                    ps.setInt(1, order.getProductId());
                    ps.setInt(2, order.getQty());
                    ps.setInt(3, order.getAccountId());
                    return ps;
                }
            }, keyHolder);
            return keyHolder.getKey().intValue();
        } catch (Exception e) {
            LOG.error("Create order failed.", e);
            throw new RuntimeException("Create order failed.");
        }
    }

    public boolean confirmOrder(Long txId, Long branchId) {
        String sql = "UPDATE dtf_demo_order SET status = 1 WHERE status = 0 AND order_id = (SELECT order_id FROM dtf_demo_order_tmp WHERE tx_id = ? AND branch_id = ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Confirm order failed.", e);
            throw new RuntimeException("Confirm order failed.");
        }
    }

    public boolean cancelOrder(Long txId, Long branchId) {
        String sql = "UPDATE dtf_demo_order SET status = 2 WHERE status = 0 AND order_id = (SELECT order_id FROM dtf_demo_order_tmp WHERE tx_id = ? AND branch_id = ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Cancel order failed.", e);
            throw new RuntimeException("Cancel order failed.");
        }
    }

    public boolean createOrderTmp(Long txId, Long branchId, int orderId) {
        String sql = "INSERT INTO dtf_demo_order_tmp (tx_id, branch_id, order_id) VALUES (?, ?, ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                    ps.setInt(3, orderId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Create order tmp failed.", e);
            throw new RuntimeException("Create order tmp failed.");
        }
    }

    public boolean deleteOrderTmp(Long txId, Long branchId) {
        String sql = "DELETE FROM dtf_demo_order_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Delete order tmp failed.", e);
            throw new RuntimeException("Delete order tmp failed.");
        }
    }

    public boolean queryOrderTmp(Long txId, Long branchId) {
        String sql = "SELECT order_id FROM dtf_demo_order_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.query(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }, new ResultSetExtractor<Boolean>() {
                @Override
                public Boolean extractData(ResultSet rs) throws SQLException, DataAccessException {
                    return null != rs && rs.next();
                }
            });
        } catch (Exception e) {
            LOG.error("Query order tmp failed.", e);
            throw new RuntimeException("Query order tmp failed.");
        }
    }
}
