package com.tencent.cloud.dtf.demo.payment.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Component;

import com.tencent.cloud.dtf.demo.payment.Payment;

@Component
public class PaymentDao {

    private static final Logger LOG = LoggerFactory.getLogger(PaymentDao.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public boolean pay(Payment payment) {
        String sql = "UPDATE dtf_demo_account SET frozen = frozen + ? WHERE account_id = ? AND balance >= frozen + ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, payment.getBalance());
                    ps.setInt(2, payment.getAccountId());
                    ps.setInt(3, payment.getBalance());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Pay failed.", e);
            throw new RuntimeException("Pay failed.");
        }
    }

    public boolean confirmPay(Payment payment) {
        String sql = "UPDATE dtf_demo_account SET frozen = frozen - ?, balance = balance - ? WHERE account_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, payment.getBalance());
                    ps.setInt(2, payment.getBalance());
                    ps.setInt(3, payment.getAccountId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Confirm Pay failed.", e);
            throw new RuntimeException("Confirm Pay failed.");
        }
    }

    public boolean cancelPay(Payment payment) {
        String sql = "UPDATE dtf_demo_account SET frozen = frozen - ? WHERE account_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, payment.getBalance());
                    ps.setInt(2, payment.getAccountId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Cancel Pay failed.", e);
            throw new RuntimeException("Cancel Pay failed.");
        }
    }

    public boolean createPaymentTmp(Long txId, Long branchId, Payment payment) {
        String sql = "INSERT INTO dtf_demo_account_tmp (tx_id, branch_id, account_id, amount) VALUES (?, ?, ?, ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                    ps.setLong(3, payment.getAccountId());
                    ps.setInt(4, payment.getBalance());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Create payment tmp failed.", e);
            throw new RuntimeException("Create payment tmp failed.");
        }
    }

    public boolean deletePaymentTmp(Long txId, Long branchId) {
        String sql = "DELETE FROM dtf_demo_account_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Delete payment tmp failed.", e);
            throw new RuntimeException("Delete payment tmp failed.");
        }
    }

    public boolean queryPaymentTmp(Long txId, Long branchId) {
        String sql = "SELECT 1 FROM dtf_demo_account_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.query(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }, new ResultSetExtractor<Boolean>() {
                @Override
                public Boolean extractData(ResultSet rs) throws SQLException, DataAccessException {
                    return null != rs && rs.next();
                }
            });
        } catch (Exception e) {
            LOG.error("Query payment tmp failed.", e);
            throw new RuntimeException("Query payment tmp failed.");
        }
    }
}