package com.tencent.cloud.dtf.demo.payment.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.tencent.cloud.dtf.demo.payment.Payment;
import com.tencent.cloud.dtf.demo.payment.dao.PaymentDao;
import com.tencent.cloud.dtf.demo.payment.proxy.PointRestTemplate;
import com.tencent.cloud.dtf.demo.payment.service.IPaymentService;

@Service
public class PaymentService implements IPaymentService {

    @Autowired
    private PaymentDao paymentDao;

    @Autowired
    private PointRestTemplate pointRestTemplate;

    @Transactional
    @Override
    public boolean pay(Long txId, Long branchId, Payment payment) {
        if (paymentDao.pay(payment) && paymentDao.createPaymentTmp(txId, branchId, payment)) {
            // 加积分
            if (pointRestTemplate.point(payment)) {
                return true;
            }
        }
        throw new RuntimeException("Pay failed.");
    }

    @Transactional
    @Override
    public boolean confirmPay(Long txId, Long branchId, Payment payment) {
        if (paymentDao.queryPaymentTmp(txId, branchId)) {
            if (paymentDao.confirmPay(payment) && paymentDao.deletePaymentTmp(txId, branchId)) {
                return true;
            }
            // 使用异常对事务进行回滚
            throw new RuntimeException("Confirm pay failed.");
        } else {
            // 幂等
            return true;
        }
    }

    @Transactional
    @Override
    public boolean cancelPay(Long txId, Long branchId, Payment payment) {
        if (paymentDao.queryPaymentTmp(txId, branchId)) {
            if (paymentDao.cancelPay(payment) && paymentDao.deletePaymentTmp(txId, branchId)) {
                return true;
            }
            // 使用异常对事务进行回滚
            throw new RuntimeException("Cancel pay failed.");
        } else {
            // 幂等
            return true;
        }
    }
}
