package com.tencent.cloud.dtf.demo.order.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.tencent.cloud.dtf.annotation.DtfTransactional;
import com.tencent.cloud.dtf.demo.order.Order;
import com.tencent.cloud.dtf.demo.order.TestOrder;
import com.tencent.cloud.dtf.demo.order.proxy.InventoryRestTemplate;
import com.tencent.cloud.dtf.demo.order.proxy.Payment;
import com.tencent.cloud.dtf.demo.order.proxy.PaymentProxy;
import com.tencent.cloud.dtf.demo.order.service.IOrderService;

@RestController
public class OrderController {

    @Autowired
    private IOrderService orderService;

    @Autowired
    private InventoryRestTemplate inventoryRestTemplate;

    @Autowired
    private PaymentProxy paymentProxy;

    @DtfTransactional(timeout = 10 * 1000)
    @RequestMapping("/order")
    public Boolean order(@RequestBody Order order) {
        // 下单
        if (orderService.order(order)) {
            // 减库存
            if (!inventoryRestTemplate.deduct(order)) {
                return false;
            }
            // 扣钱 - 加积分
            Payment payment = new Payment();
            payment.setAccountId(order.getAccountId());
            payment.setBalance(order.getQty());
            return paymentProxy.pay(payment);
        } else {
            throw new RuntimeException("Create Order Failed.");
        }
    }

    @DtfTransactional(timeout = 10 * 1000)
    @RequestMapping("/test")
    public Boolean test(@RequestBody TestOrder testOrder) {
        return orderService.test(testOrder);
    }
}
