package com.tencent.cloud.dtf.demo.order.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Component;

import com.tencent.cloud.dtf.demo.order.Order;
import com.tencent.cloud.dtf.demo.order.TestOrder;

@Component
public class OrderDao {
    private static final Logger LOG = LoggerFactory.getLogger(OrderDao.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public int createOrder(Order order) {
        String sql = "INSERT INTO `dtf_demo_order` (`product_id`, qty, account_id) VALUES (?, ?, ?)";
        try {
            GeneratedKeyHolder keyHolder = new GeneratedKeyHolder();
            jdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection con) throws SQLException {
                    PreparedStatement ps = con.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
                    ps.setInt(1, order.getProductId());
                    ps.setInt(2, order.getQty());
                    ps.setInt(3, order.getAccountId());
                    return ps;
                }
            }, keyHolder);
            return keyHolder.getKey().intValue();
        } catch (Exception e) {
            LOG.error("Create order failed.", e);
            throw new RuntimeException("Create order failed.");
        }
    }

    public int test(TestOrder testOrder) {
        String sql = "UPDATE `order_info`          SET `status`=?,                                                                                                               `remark`=?,                                                                                                               `create_time`=?,                                                                                                               `update_time`=?,                                                                                                               `creator`=?,                                                                                                               `editor`=?          WHERE `order_id` = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, testOrder.getStatus());
                    ps.setString(2, testOrder.getRemark());
                    ps.setString(3, testOrder.getCreateTime());
                    ps.setString(4, testOrder.getUpdateTime());
                    ps.setString(5, testOrder.getCreator());
                    ps.setString(6, testOrder.getEditor());
                    ps.setLong(7, testOrder.getOrderId());
                }
            });
        } catch (Exception e) {
            LOG.error("Failed execute update.", e);
            throw new RuntimeException("fail");
        }
    }
}
