package com.tencent.cloud.dtf.demo.point.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementSetter;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Component;

import com.tencent.cloud.dtf.demo.point.Point;

@Component
public class PointDao {

    private static final Logger LOG = LoggerFactory.getLogger(PointDao.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public boolean point(Point point) {
        String sql = "UPDATE dtf_demo_point SET frozen = frozen + ? WHERE account_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, point.getPoint());
                    ps.setInt(2, point.getAccountId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Record point failed.", e);
            throw new RuntimeException("Record point failed.");
        }
    }

    public boolean confirmPoint(Point point) {
        String sql = "UPDATE dtf_demo_point SET frozen = frozen - ?, point = point + ? WHERE account_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, point.getPoint());
                    ps.setInt(2, point.getPoint());
                    ps.setInt(3, point.getAccountId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Confirm point failed.", e);
            throw new RuntimeException("Confirm point failed.");
        }
    }

    public boolean cancelPoint(Point point) {
        String sql = "UPDATE dtf_demo_point SET frozen = frozen - ? WHERE account_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setInt(1, point.getPoint());
                    ps.setInt(2, point.getAccountId());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Cancel point failed.", e);
            throw new RuntimeException("Cancel point failed.");
        }
    }

    public boolean createPointTmp(Long txId, Long branchId, Point point) {
        String sql = "INSERT INTO dtf_demo_point_tmp (tx_id, branch_id, account_id, point) VALUES (?, ?, ?, ?)";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                    ps.setInt(3, point.getAccountId());
                    ps.setInt(4, point.getPoint());
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Create point tmp failed.", e);
            throw new RuntimeException("Create point tmp failed.");
        }
    }

    public boolean deletePointTmp(Long txId, Long branchId, Point point) {
        String sql = "DELETE FROM dtf_demo_point_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.update(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }) > 0;
        } catch (Exception e) {
            LOG.error("Delete point tmp failed.", e);
            throw new RuntimeException("Delete point tmp failed.");
        }
    }

    public boolean queryPointTmp(Long txId, Long branchId, Point point) {
        String sql = "SELECT 1 FROM dtf_demo_point_tmp WHERE tx_id = ? AND branch_id = ?";
        try {
            return jdbcTemplate.query(sql, new PreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps) throws SQLException {
                    ps.setLong(1, txId);
                    ps.setLong(2, branchId);
                }
            }, new ResultSetExtractor<Boolean>() {
                @Override
                public Boolean extractData(ResultSet rs) throws SQLException, DataAccessException {
                    return null != rs && rs.next();
                }
            });
        } catch (Exception e) {
            LOG.error("Query point tmp failed.", e);
            throw new RuntimeException("Query point tmp failed.");
        }
    }
}
